package util

import (
	"fmt"
	"io"
	"io/fs"
	"os"
	"os/exec"
	"path/filepath"
	"sort"
	"strings"
)

// 检查文件是否存在，如果存在则删除它，不返回错误
func DeleteFileIfExists(filePath string) {
	// 使用Stat函数检查文件是否存在
	fileInfo, err := os.Stat(filePath)
	if err != nil {
		// 如果Stat返回错误，且错误是因为文件不存在，则忽略
		if os.IsNotExist(err) {
			fmt.Println("文件不存在:", filePath)
			return
		}
		// 如果错误不是文件不存在，则记录或处理错误
		fmt.Println("获取文件信息时出错:", err)
		// 这里可以根据需要添加更多的错误处理逻辑，但按照要求我们不返回错误
		return
	}

	// 如果文件存在，则检查是否为文件（虽然大多数情况下会是文件，但最好还是检查一下）
	if !fileInfo.IsDir() {
		// 调用Remove函数删除文件
		err := os.Remove(filePath)
		if err != nil {
			// 删除文件时出错，记录或处理错误
			fmt.Println("删除文件时出错:", err)
			// 这里可以根据需要添加更多的错误处理逻辑，但按照要求我们不返回错误
		} else {
			fmt.Println("文件已删除:", filePath)
		}
	} else {
		fmt.Println(filePath, "是一个目录，不是文件")
	}
}

// 将指定的目录和子目录压缩成 ZIP 文件
func ZipDir(src string, dest string, dirs []string) error {
	// 创建 zip 命令参数
	args := []string{"-r", dest}
	for _, dir := range dirs {
		args = append(args, dir)
	}

	// 创建 zip 命令
	cmd := exec.Command("zip", args...)

	// 设置工作目录
	cmd.Dir = src

	// 执行 zip 命令并获取输出
	output, err := cmd.CombinedOutput()
	if err != nil {
		return fmt.Errorf("执行压缩命令【%v】报错【%v】输出【%v】", args, err, output)
	}

	fmt.Printf("执行压缩命令【%v】成功", args)
	return nil
}

func GetFileSize(filePath string) (int, error) {
	fileInfo, err := os.Stat(filePath)
	if err != nil {
		return 0, err
	}
	return int(fileInfo.Size()), nil
}

func CheckSoFilesInDirectory(dirPath string) (bool, []string) {
	hasSoFile := false
	soFilePaths := make([]string, 0)
	if _, err := os.Stat(dirPath); os.IsNotExist(err) {
		fmt.Println("Directory does not exist:", dirPath)
		return false, soFilePaths
	}

	err := filepath.Walk(dirPath, func(path string, info fs.FileInfo, err error) error {
		if err != nil {
			return err
		}

		if info.IsDir() {
			return nil
		}

		if filepath.Ext(path) == ".so" {
			hasSoFile = true
			soFilePaths = append(soFilePaths, path)
		}
		return nil
	})

	if err != nil {
		fmt.Println("Error walking the directory:", err)
		os.Exit(1)
	}

	return hasSoFile, soFilePaths
}

func GetFirstLevelSubdirectories(dirPath string) []string {
	dirEntries, _ := os.ReadDir(dirPath)
	subdirectories := make([]string, 0)
	for _, entry := range dirEntries {
		if entry.IsDir() {
			subdirectories = append(subdirectories, entry.Name())
		}
	}
	return subdirectories
}

func GetFileNameWithoutExtension(absPath string) string {
	file := filepath.Base(absPath)
	ext := filepath.Ext(file)
	return strings.TrimSuffix(file, ext)
}

func GetSoFilePaths(dir string) []string {
	var paths []string
	err := filepath.Walk(dir, func(path string, info os.FileInfo, err error) error {
		if err == nil && !info.IsDir() && filepath.Ext(path) == ".so" {
			absPath, _ := filepath.Abs(path)
			paths = append(paths, absPath)
		}
		return nil
	})
	if err != nil {
		fmt.Println("Error:", err)
		return nil
	}
	return paths
}

// CreateDir 创建目录
func CreateDir(directory string) {
	if _, err := os.Stat(directory); os.IsNotExist(err) {
		_ = os.MkdirAll(directory, os.ModePerm)
	}
}

// CreateParentDir 存在不创建，不存在则创建父目录
func CreateParentDir(filePath string) error {
	if err := os.MkdirAll(filepath.Dir(filePath), os.ModePerm); err != nil {
		return err
	}
	return nil
}

// RemoveDir 递归删除目录及其下的所有文件和子目录
func RemoveDir(dirPath string) error {
	// 打开目录
	dir, err := os.Open(dirPath)
	if err != nil {
		return err
	}
	defer dir.Close()
	// 读取目录下的文件和子目录
	fileInfos, err := dir.Readdir(-1)
	if err != nil {
		return err
	}
	// 遍历文件和子目录
	for _, fileInfo := range fileInfos {
		path := filepath.Join(dirPath, fileInfo.Name())

		if fileInfo.IsDir() {
			// 如果是子目录，递归调用removeDir删除子目录及其下的文件和子目录
			if err = RemoveDir(path); err != nil {
				return err
			}
		} else {
			// 如果是文件，直接删除文件
			if err = os.Remove(path); err != nil {
				return err
			}
		}
	}
	// 删除目录本身
	if err = os.Remove(dirPath); err != nil {
		return err
	}
	return nil
}

// RemoveSubFiles 递归删除目录及其下的所有文件和子目录
func RemoveSubFiles(dirPath string) error {
	// 打开目录
	dir, err := os.Open(dirPath)
	if err != nil {
		return err
	}
	defer dir.Close()
	// 读取目录下的文件和子目录
	fileInfos, err := dir.Readdir(-1)
	if err != nil {
		return err
	}
	// 遍历文件和子目录
	for _, fileInfo := range fileInfos {
		path := filepath.Join(dirPath, fileInfo.Name())

		if fileInfo.IsDir() {
			// 如果是子目录，递归调用removeDir删除子目录及其下的文件和子目录
			if err = RemoveDir(path); err != nil {
				return err
			}
		} else {
			// 如果是文件，直接删除文件
			if err = os.Remove(path); err != nil {
				return err
			}
		}
	}
	return nil
}

func WriteFile(sourceContent string, targetFilePath string) error {
	if err := CreateFile(targetFilePath); err != nil {
		return err
	}
	if err := os.WriteFile(targetFilePath, []byte(sourceContent), 0644); err != nil {
		return err
	}
	return nil
}

// CreateFile 存在则覆盖，不存在则创建文件
func CreateFile(filePath string) error {
	if err := CreateParentDir(filePath); err != nil {
		return err
	}
	// 创建文件，如果文件已存在则覆盖
	file, err := os.Create(filePath)
	if err != nil {
		return err
	}
	defer func(file *os.File) {
		err := file.Close()
		if err != nil {

		}
	}(file)
	return nil
}
func ReadFile(filePath string) (string, error) {
	// 1 打开文件
	file, err := os.Open(filePath)
	if err != nil {
		return "", err
	}
	defer file.Close()

	// 2 读取文件内容
	content, err := io.ReadAll(file)
	if err != nil {
		return "", err
	}
	return string(content), err
}

func ListAbsolutePathWithSuffixAndSort(dir string, suffix string) ([]string, error) {
	var result []string
	if !strings.HasSuffix(dir, "/") {
		dir = dir + "/"
	}
	files, err := os.ReadDir(dir)
	if err != nil {
		return nil, err
	}
	for _, file := range files {
		if strings.HasSuffix(file.Name(), suffix) {
			result = append(result, dir+file.Name())
		}
	}
	// 根据文件名进行升序排序
	sort.Slice(result, func(i, j int) bool {
		return filepath.Base(result[i]) < filepath.Base(result[j])
	})
	return result, nil
}

func DeleteFile(path string) error {
	// 检查文件是否存在
	if _, err := os.Stat(path); err == nil {
		// 文件存在，执行删除操作
		err = os.Remove(path)
		if err != nil {
			return err
		}
	}
	return nil
}

// WriteStringToFile 将字符串写入指定的文件路径
func WriteStringToFile(content string, filePath string) {
	// 方法一：使用os包
	file, err := os.Create(filePath)
	if err != nil {
		fmt.Println("创建文件失败:", err)
		return
	}
	defer file.Close()

	_, err = file.WriteString(content)
	if err != nil {
		fmt.Println("写入文件失败:", err)
		return
	}
	fmt.Println("字符串成功写入文件（os包）")
}
